@@ -1,3 +1,4 @@
+use anyhow::Context;
 use ginepro::LoadBalancedChannel;
 
 use shared_proto::pb::{echo_client::EchoClient, EchoRequest};
@@ -7,9 +8,10 @@ async fn main() -> Result<(), Box<dyn std::error::Error>> {
     // Create a load balanced channel that connects to localhost:5000.
     // Here the service discovery will update the set of servers every 5 seconds.
     let channel = LoadBalancedChannel::builder(("localhost", 5000_u16))
-        .await?
         .dns_probe_interval(std::time::Duration::from_secs(5))
-        .channel();
+        .channel()
+        .await
+        .context("failed to construct LoadBalancedChannel")?;
 
     // Use the channel created above to drive the communication in EchoClient.
     let mut client = EchoClient::new(channel);
