@@ -17,10 +17,10 @@ async fn main() -> Result<(), Box<dyn std::error::Error>> {
     let tls = ClientTlsConfig::new().ca_certificate(ca);
 
     let channel = LoadBalancedChannel::builder(("localhost", 5000_u16))
-        .await?
         .with_tls(tls)
         .dns_probe_interval(std::time::Duration::from_secs(5))
-        .channel();
+        .channel()
+        .await?;
 
     let mut client = EchoClient::new(channel);
 
